/****************************************************************************
** Meta object code from reading C++ file 'qlowenergycontroller.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/qlowenergycontroller.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlowenergycontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QLowEnergyController_t {
    QByteArrayData data[41];
    char stringdata0[640];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QLowEnergyController_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QLowEnergyController_t qt_meta_stringdata_QLowEnergyController = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QLowEnergyController"
QT_MOC_LITERAL(1, 21, 9), // "connected"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 12), // "disconnected"
QT_MOC_LITERAL(4, 45, 12), // "stateChanged"
QT_MOC_LITERAL(5, 58, 37), // "QLowEnergyController::Control..."
QT_MOC_LITERAL(6, 96, 5), // "state"
QT_MOC_LITERAL(7, 102, 5), // "error"
QT_MOC_LITERAL(8, 108, 27), // "QLowEnergyController::Error"
QT_MOC_LITERAL(9, 136, 8), // "newError"
QT_MOC_LITERAL(10, 145, 17), // "serviceDiscovered"
QT_MOC_LITERAL(11, 163, 14), // "QBluetoothUuid"
QT_MOC_LITERAL(12, 178, 10), // "newService"
QT_MOC_LITERAL(13, 189, 17), // "discoveryFinished"
QT_MOC_LITERAL(14, 207, 17), // "connectionUpdated"
QT_MOC_LITERAL(15, 225, 30), // "QLowEnergyConnectionParameters"
QT_MOC_LITERAL(16, 256, 10), // "parameters"
QT_MOC_LITERAL(17, 267, 5), // "Error"
QT_MOC_LITERAL(18, 273, 7), // "NoError"
QT_MOC_LITERAL(19, 281, 12), // "UnknownError"
QT_MOC_LITERAL(20, 294, 24), // "UnknownRemoteDeviceError"
QT_MOC_LITERAL(21, 319, 12), // "NetworkError"
QT_MOC_LITERAL(22, 332, 28), // "InvalidBluetoothAdapterError"
QT_MOC_LITERAL(23, 361, 15), // "ConnectionError"
QT_MOC_LITERAL(24, 377, 16), // "AdvertisingError"
QT_MOC_LITERAL(25, 394, 21), // "RemoteHostClosedError"
QT_MOC_LITERAL(26, 416, 18), // "AuthorizationError"
QT_MOC_LITERAL(27, 435, 15), // "ControllerState"
QT_MOC_LITERAL(28, 451, 16), // "UnconnectedState"
QT_MOC_LITERAL(29, 468, 15), // "ConnectingState"
QT_MOC_LITERAL(30, 484, 14), // "ConnectedState"
QT_MOC_LITERAL(31, 499, 16), // "DiscoveringState"
QT_MOC_LITERAL(32, 516, 15), // "DiscoveredState"
QT_MOC_LITERAL(33, 532, 12), // "ClosingState"
QT_MOC_LITERAL(34, 545, 16), // "AdvertisingState"
QT_MOC_LITERAL(35, 562, 17), // "RemoteAddressType"
QT_MOC_LITERAL(36, 580, 13), // "PublicAddress"
QT_MOC_LITERAL(37, 594, 13), // "RandomAddress"
QT_MOC_LITERAL(38, 608, 4), // "Role"
QT_MOC_LITERAL(39, 613, 11), // "CentralRole"
QT_MOC_LITERAL(40, 625, 14) // "PeripheralRole"

    },
    "QLowEnergyController\0connected\0\0"
    "disconnected\0stateChanged\0"
    "QLowEnergyController::ControllerState\0"
    "state\0error\0QLowEnergyController::Error\0"
    "newError\0serviceDiscovered\0QBluetoothUuid\0"
    "newService\0discoveryFinished\0"
    "connectionUpdated\0QLowEnergyConnectionParameters\0"
    "parameters\0Error\0NoError\0UnknownError\0"
    "UnknownRemoteDeviceError\0NetworkError\0"
    "InvalidBluetoothAdapterError\0"
    "ConnectionError\0AdvertisingError\0"
    "RemoteHostClosedError\0AuthorizationError\0"
    "ControllerState\0UnconnectedState\0"
    "ConnectingState\0ConnectedState\0"
    "DiscoveringState\0DiscoveredState\0"
    "ClosingState\0AdvertisingState\0"
    "RemoteAddressType\0PublicAddress\0"
    "RandomAddress\0Role\0CentralRole\0"
    "PeripheralRole"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QLowEnergyController[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       4,   64, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,
       3,    0,   50,    2, 0x06 /* Public */,
       4,    1,   51,    2, 0x06 /* Public */,
       7,    1,   54,    2, 0x06 /* Public */,
      10,    1,   57,    2, 0x06 /* Public */,
      13,    0,   60,    2, 0x06 /* Public */,
      14,    1,   61,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15,   16,

 // enums: name, alias, flags, count, data
      17,   17, 0x0,    9,   84,
      27,   27, 0x0,    7,  102,
      35,   35, 0x0,    2,  116,
      38,   38, 0x0,    2,  120,

 // enum data: key, value
      18, uint(QLowEnergyController::NoError),
      19, uint(QLowEnergyController::UnknownError),
      20, uint(QLowEnergyController::UnknownRemoteDeviceError),
      21, uint(QLowEnergyController::NetworkError),
      22, uint(QLowEnergyController::InvalidBluetoothAdapterError),
      23, uint(QLowEnergyController::ConnectionError),
      24, uint(QLowEnergyController::AdvertisingError),
      25, uint(QLowEnergyController::RemoteHostClosedError),
      26, uint(QLowEnergyController::AuthorizationError),
      28, uint(QLowEnergyController::UnconnectedState),
      29, uint(QLowEnergyController::ConnectingState),
      30, uint(QLowEnergyController::ConnectedState),
      31, uint(QLowEnergyController::DiscoveringState),
      32, uint(QLowEnergyController::DiscoveredState),
      33, uint(QLowEnergyController::ClosingState),
      34, uint(QLowEnergyController::AdvertisingState),
      36, uint(QLowEnergyController::PublicAddress),
      37, uint(QLowEnergyController::RandomAddress),
      39, uint(QLowEnergyController::CentralRole),
      40, uint(QLowEnergyController::PeripheralRole),

       0        // eod
};

void QLowEnergyController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QLowEnergyController *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->connected(); break;
        case 1: _t->disconnected(); break;
        case 2: _t->stateChanged((*reinterpret_cast< QLowEnergyController::ControllerState(*)>(_a[1]))); break;
        case 3: _t->error((*reinterpret_cast< QLowEnergyController::Error(*)>(_a[1]))); break;
        case 4: _t->serviceDiscovered((*reinterpret_cast< const QBluetoothUuid(*)>(_a[1]))); break;
        case 5: _t->discoveryFinished(); break;
        case 6: _t->connectionUpdated((*reinterpret_cast< const QLowEnergyConnectionParameters(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyController::ControllerState >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyController::Error >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothUuid >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QLowEnergyController::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyController::connected)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QLowEnergyController::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyController::disconnected)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QLowEnergyController::*)(QLowEnergyController::ControllerState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyController::stateChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QLowEnergyController::*)(QLowEnergyController::Error );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyController::error)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QLowEnergyController::*)(const QBluetoothUuid & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyController::serviceDiscovered)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QLowEnergyController::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyController::discoveryFinished)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QLowEnergyController::*)(const QLowEnergyConnectionParameters & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyController::connectionUpdated)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QLowEnergyController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QLowEnergyController.data,
    qt_meta_data_QLowEnergyController,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QLowEnergyController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLowEnergyController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QLowEnergyController.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QLowEnergyController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QLowEnergyController::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QLowEnergyController::disconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QLowEnergyController::stateChanged(QLowEnergyController::ControllerState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QLowEnergyController::error(QLowEnergyController::Error _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QLowEnergyController::serviceDiscovered(const QBluetoothUuid & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QLowEnergyController::discoveryFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QLowEnergyController::connectionUpdated(const QLowEnergyConnectionParameters & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
