/****************************************************************************
** Meta object code from reading C++ file 'qlowenergyservice.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/qlowenergyservice.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qlowenergyservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QLowEnergyService_t {
    QByteArrayData data[37];
    char stringdata0[604];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QLowEnergyService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QLowEnergyService_t qt_meta_stringdata_QLowEnergyService = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QLowEnergyService"
QT_MOC_LITERAL(1, 18, 12), // "stateChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 31), // "QLowEnergyService::ServiceState"
QT_MOC_LITERAL(4, 64, 8), // "newState"
QT_MOC_LITERAL(5, 73, 21), // "characteristicChanged"
QT_MOC_LITERAL(6, 95, 24), // "QLowEnergyCharacteristic"
QT_MOC_LITERAL(7, 120, 4), // "info"
QT_MOC_LITERAL(8, 125, 5), // "value"
QT_MOC_LITERAL(9, 131, 18), // "characteristicRead"
QT_MOC_LITERAL(10, 150, 21), // "characteristicWritten"
QT_MOC_LITERAL(11, 172, 14), // "descriptorRead"
QT_MOC_LITERAL(12, 187, 20), // "QLowEnergyDescriptor"
QT_MOC_LITERAL(13, 208, 17), // "descriptorWritten"
QT_MOC_LITERAL(14, 226, 5), // "error"
QT_MOC_LITERAL(15, 232, 31), // "QLowEnergyService::ServiceError"
QT_MOC_LITERAL(16, 264, 11), // "ServiceType"
QT_MOC_LITERAL(17, 276, 14), // "PrimaryService"
QT_MOC_LITERAL(18, 291, 15), // "IncludedService"
QT_MOC_LITERAL(19, 307, 12), // "ServiceError"
QT_MOC_LITERAL(20, 320, 7), // "NoError"
QT_MOC_LITERAL(21, 328, 14), // "OperationError"
QT_MOC_LITERAL(22, 343, 24), // "CharacteristicWriteError"
QT_MOC_LITERAL(23, 368, 20), // "DescriptorWriteError"
QT_MOC_LITERAL(24, 389, 12), // "UnknownError"
QT_MOC_LITERAL(25, 402, 23), // "CharacteristicReadError"
QT_MOC_LITERAL(26, 426, 19), // "DescriptorReadError"
QT_MOC_LITERAL(27, 446, 12), // "ServiceState"
QT_MOC_LITERAL(28, 459, 14), // "InvalidService"
QT_MOC_LITERAL(29, 474, 17), // "DiscoveryRequired"
QT_MOC_LITERAL(30, 492, 19), // "DiscoveringServices"
QT_MOC_LITERAL(31, 512, 17), // "ServiceDiscovered"
QT_MOC_LITERAL(32, 530, 12), // "LocalService"
QT_MOC_LITERAL(33, 543, 9), // "WriteMode"
QT_MOC_LITERAL(34, 553, 17), // "WriteWithResponse"
QT_MOC_LITERAL(35, 571, 20), // "WriteWithoutResponse"
QT_MOC_LITERAL(36, 592, 11) // "WriteSigned"

    },
    "QLowEnergyService\0stateChanged\0\0"
    "QLowEnergyService::ServiceState\0"
    "newState\0characteristicChanged\0"
    "QLowEnergyCharacteristic\0info\0value\0"
    "characteristicRead\0characteristicWritten\0"
    "descriptorRead\0QLowEnergyDescriptor\0"
    "descriptorWritten\0error\0"
    "QLowEnergyService::ServiceError\0"
    "ServiceType\0PrimaryService\0IncludedService\0"
    "ServiceError\0NoError\0OperationError\0"
    "CharacteristicWriteError\0DescriptorWriteError\0"
    "UnknownError\0CharacteristicReadError\0"
    "DescriptorReadError\0ServiceState\0"
    "InvalidService\0DiscoveryRequired\0"
    "DiscoveringServices\0ServiceDiscovered\0"
    "LocalService\0WriteMode\0WriteWithResponse\0"
    "WriteWithoutResponse\0WriteSigned"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QLowEnergyService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       4,   80, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       5,    2,   52,    2, 0x06 /* Public */,
       9,    2,   57,    2, 0x06 /* Public */,
      10,    2,   62,    2, 0x06 /* Public */,
      11,    2,   67,    2, 0x06 /* Public */,
      13,    2,   72,    2, 0x06 /* Public */,
      14,    1,   77,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6, QMetaType::QByteArray,    7,    8,
    QMetaType::Void, 0x80000000 | 6, QMetaType::QByteArray,    7,    8,
    QMetaType::Void, 0x80000000 | 6, QMetaType::QByteArray,    7,    8,
    QMetaType::Void, 0x80000000 | 12, QMetaType::QByteArray,    7,    8,
    QMetaType::Void, 0x80000000 | 12, QMetaType::QByteArray,    7,    8,
    QMetaType::Void, 0x80000000 | 15,   14,

 // enums: name, alias, flags, count, data
      16,   16, 0x0,    2,  100,
      19,   19, 0x0,    7,  104,
      27,   27, 0x0,    5,  118,
      33,   33, 0x0,    3,  128,

 // enum data: key, value
      17, uint(QLowEnergyService::PrimaryService),
      18, uint(QLowEnergyService::IncludedService),
      20, uint(QLowEnergyService::NoError),
      21, uint(QLowEnergyService::OperationError),
      22, uint(QLowEnergyService::CharacteristicWriteError),
      23, uint(QLowEnergyService::DescriptorWriteError),
      24, uint(QLowEnergyService::UnknownError),
      25, uint(QLowEnergyService::CharacteristicReadError),
      26, uint(QLowEnergyService::DescriptorReadError),
      28, uint(QLowEnergyService::InvalidService),
      29, uint(QLowEnergyService::DiscoveryRequired),
      30, uint(QLowEnergyService::DiscoveringServices),
      31, uint(QLowEnergyService::ServiceDiscovered),
      32, uint(QLowEnergyService::LocalService),
      34, uint(QLowEnergyService::WriteWithResponse),
      35, uint(QLowEnergyService::WriteWithoutResponse),
      36, uint(QLowEnergyService::WriteSigned),

       0        // eod
};

void QLowEnergyService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QLowEnergyService *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->stateChanged((*reinterpret_cast< QLowEnergyService::ServiceState(*)>(_a[1]))); break;
        case 1: _t->characteristicChanged((*reinterpret_cast< const QLowEnergyCharacteristic(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 2: _t->characteristicRead((*reinterpret_cast< const QLowEnergyCharacteristic(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 3: _t->characteristicWritten((*reinterpret_cast< const QLowEnergyCharacteristic(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 4: _t->descriptorRead((*reinterpret_cast< const QLowEnergyDescriptor(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 5: _t->descriptorWritten((*reinterpret_cast< const QLowEnergyDescriptor(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 6: _t->error((*reinterpret_cast< QLowEnergyService::ServiceError(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyService::ServiceState >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyCharacteristic >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyCharacteristic >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyCharacteristic >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyDescriptor >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyDescriptor >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QLowEnergyService::ServiceError >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QLowEnergyService::*)(QLowEnergyService::ServiceState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyService::stateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QLowEnergyService::*)(const QLowEnergyCharacteristic & , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyService::characteristicChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QLowEnergyService::*)(const QLowEnergyCharacteristic & , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyService::characteristicRead)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QLowEnergyService::*)(const QLowEnergyCharacteristic & , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyService::characteristicWritten)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QLowEnergyService::*)(const QLowEnergyDescriptor & , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyService::descriptorRead)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QLowEnergyService::*)(const QLowEnergyDescriptor & , const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyService::descriptorWritten)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QLowEnergyService::*)(QLowEnergyService::ServiceError );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QLowEnergyService::error)) {
                *result = 6;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QLowEnergyService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QLowEnergyService.data,
    qt_meta_data_QLowEnergyService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QLowEnergyService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QLowEnergyService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QLowEnergyService.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QLowEnergyService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QLowEnergyService::stateChanged(QLowEnergyService::ServiceState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QLowEnergyService::characteristicChanged(const QLowEnergyCharacteristic & _t1, const QByteArray & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QLowEnergyService::characteristicRead(const QLowEnergyCharacteristic & _t1, const QByteArray & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QLowEnergyService::characteristicWritten(const QLowEnergyCharacteristic & _t1, const QByteArray & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QLowEnergyService::descriptorRead(const QLowEnergyDescriptor & _t1, const QByteArray & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QLowEnergyService::descriptorWritten(const QLowEnergyDescriptor & _t1, const QByteArray & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QLowEnergyService::error(QLowEnergyService::ServiceError _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
