/****************************************************************************
** Meta object code from reading C++ file 'service_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/service_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'service_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgBluezServiceInterface_t {
    QByteArrayData data[11];
    char stringdata0[155];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgBluezServiceInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgBluezServiceInterface_t qt_meta_stringdata_OrgBluezServiceInterface = {
    {
QT_MOC_LITERAL(0, 0, 24), // "OrgBluezServiceInterface"
QT_MOC_LITERAL(1, 25, 9), // "AddRecord"
QT_MOC_LITERAL(2, 35, 23), // "QDBusPendingReply<uint>"
QT_MOC_LITERAL(3, 59, 0), // ""
QT_MOC_LITERAL(4, 60, 3), // "in0"
QT_MOC_LITERAL(5, 64, 19), // "CancelAuthorization"
QT_MOC_LITERAL(6, 84, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(7, 104, 12), // "RemoveRecord"
QT_MOC_LITERAL(8, 117, 20), // "RequestAuthorization"
QT_MOC_LITERAL(9, 138, 3), // "in1"
QT_MOC_LITERAL(10, 142, 12) // "UpdateRecord"

    },
    "OrgBluezServiceInterface\0AddRecord\0"
    "QDBusPendingReply<uint>\0\0in0\0"
    "CancelAuthorization\0QDBusPendingReply<>\0"
    "RemoveRecord\0RequestAuthorization\0in1\0"
    "UpdateRecord"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgBluezServiceInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   39,    3, 0x0a /* Public */,
       5,    0,   42,    3, 0x0a /* Public */,
       7,    1,   43,    3, 0x0a /* Public */,
       8,    2,   46,    3, 0x0a /* Public */,
      10,    2,   51,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2, QMetaType::QString,    4,
    0x80000000 | 6,
    0x80000000 | 6, QMetaType::UInt,    4,
    0x80000000 | 6, QMetaType::QString, QMetaType::UInt,    4,    9,
    0x80000000 | 6, QMetaType::UInt, QMetaType::QString,    4,    9,

       0        // eod
};

void OrgBluezServiceInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgBluezServiceInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusPendingReply<uint> _r = _t->AddRecord((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<uint>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->CancelAuthorization();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->RemoveRecord((*reinterpret_cast< uint(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->RequestAuthorization((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->UpdateRecord((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OrgBluezServiceInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_OrgBluezServiceInterface.data,
    qt_meta_data_OrgBluezServiceInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgBluezServiceInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezServiceInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgBluezServiceInterface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgBluezServiceInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
