/****************************************************************************
** Meta object code from reading C++ file 'profile1context_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/profile1context_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'profile1context_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgBluezProfile1ContextInterface_t {
    QByteArrayData data[14];
    char stringdata0[757];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgBluezProfile1ContextInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgBluezProfile1ContextInterface_t qt_meta_stringdata_OrgBluezProfile1ContextInterface = {
    {
QT_MOC_LITERAL(0, 0, 32), // "OrgBluezProfile1ContextInterface"
QT_MOC_LITERAL(1, 33, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 49, 18), // "org.bluez.Profile1"
QT_MOC_LITERAL(3, 68, 19), // "D-Bus Introspection"
QT_MOC_LITERAL(4, 88, 557), // "  <interface name=\"org.bluez..."
QT_MOC_LITERAL(5, 600, 13), // "newConnection"
QT_MOC_LITERAL(6, 614, 0), // ""
QT_MOC_LITERAL(7, 615, 23), // "QDBusUnixFileDescriptor"
QT_MOC_LITERAL(8, 639, 2), // "fd"
QT_MOC_LITERAL(9, 642, 13), // "NewConnection"
QT_MOC_LITERAL(10, 656, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(11, 672, 20), // "RequestDisconnection"
QT_MOC_LITERAL(12, 693, 7), // "Release"
QT_MOC_LITERAL(13, 701, 9) // "Q_NOREPLY"

    },
    "OrgBluezProfile1ContextInterface\0"
    "D-Bus Interface\0org.bluez.Profile1\0"
    "D-Bus Introspection\0"
    "  <interface name=\"org.bluez.Profile1\">\n    <method name=\"NewConne"
    "ction\">\n      <arg direction=\"in\" type=\"o\"/>\n      <arg directi"
    "on=\"in\" type=\"h\"/>\n      <arg direction=\"in\" type=\"a{sv}\"/>\n"
    "      <annotation value=\"QVariantMap\" name=\"org.qtproject.QtDBus.Qt"
    "TypeName.In2\"/>\n    </method>\n    <method name=\"RequestDisconnecti"
    "on\">\n      <arg direction=\"in\" type=\"o\"/>\n    </method>\n    <m"
    "ethod name=\"Release\">\n      <annotation value=\"true\" name=\"org.f"
    "reedesktop.DBus.Method.NoReply\"/>\n    </method>\n  </interface>\n\0"
    "newConnection\0\0QDBusUnixFileDescriptor\0"
    "fd\0NewConnection\0QDBusObjectPath\0"
    "RequestDisconnection\0Release\0Q_NOREPLY"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgBluezProfile1ContextInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       4,   18, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    1,   38,    6, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    3,   41,    6, 0x0a /* Public */,
      11,    1,   48,    6, 0x0a /* Public */,
      12,    0,   51,   13, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 7,    8,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 7, QMetaType::QVariantMap,    6,    6,    6,
    QMetaType::Void, 0x80000000 | 10,    6,
    QMetaType::Void,

       0        // eod
};

void OrgBluezProfile1ContextInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgBluezProfile1ContextInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->newConnection((*reinterpret_cast< const QDBusUnixFileDescriptor(*)>(_a[1]))); break;
        case 1: _t->NewConnection((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< const QDBusUnixFileDescriptor(*)>(_a[2])),(*reinterpret_cast< const QVariantMap(*)>(_a[3]))); break;
        case 2: _t->RequestDisconnection((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 3: _t->Release(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusUnixFileDescriptor >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusUnixFileDescriptor >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (OrgBluezProfile1ContextInterface::*)(const QDBusUnixFileDescriptor & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OrgBluezProfile1ContextInterface::newConnection)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OrgBluezProfile1ContextInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_OrgBluezProfile1ContextInterface.data,
    qt_meta_data_OrgBluezProfile1ContextInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgBluezProfile1ContextInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezProfile1ContextInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgBluezProfile1ContextInterface.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int OrgBluezProfile1ContextInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void OrgBluezProfile1ContextInterface::newConnection(const QDBusUnixFileDescriptor & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
