/****************************************************************************
** Meta object code from reading C++ file 'qbluetoothlocaldevice_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/qbluetoothlocaldevice_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbluetoothlocaldevice_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QBluetoothLocalDevicePrivate_t {
    QByteArrayData data[47];
    char stringdata0[684];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QBluetoothLocalDevicePrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QBluetoothLocalDevicePrivate_t qt_meta_stringdata_QBluetoothLocalDevicePrivate = {
    {
QT_MOC_LITERAL(0, 0, 28), // "QBluetoothLocalDevicePrivate"
QT_MOC_LITERAL(1, 29, 9), // "Authorize"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(4, 56, 3), // "in0"
QT_MOC_LITERAL(5, 60, 3), // "in1"
QT_MOC_LITERAL(6, 64, 6), // "Cancel"
QT_MOC_LITERAL(7, 71, 17), // "ConfirmModeChange"
QT_MOC_LITERAL(8, 89, 14), // "DisplayPasskey"
QT_MOC_LITERAL(9, 104, 3), // "in2"
QT_MOC_LITERAL(10, 108, 7), // "Release"
QT_MOC_LITERAL(11, 116, 14), // "RequestPasskey"
QT_MOC_LITERAL(12, 131, 19), // "RequestConfirmation"
QT_MOC_LITERAL(13, 151, 14), // "RequestPinCode"
QT_MOC_LITERAL(14, 166, 16), // "pairingCompleted"
QT_MOC_LITERAL(15, 183, 24), // "QDBusPendingCallWatcher*"
QT_MOC_LITERAL(16, 208, 15), // "PropertyChanged"
QT_MOC_LITERAL(17, 224, 12), // "QDBusVariant"
QT_MOC_LITERAL(18, 237, 16), // "_q_deviceCreated"
QT_MOC_LITERAL(19, 254, 6), // "device"
QT_MOC_LITERAL(20, 261, 16), // "_q_deviceRemoved"
QT_MOC_LITERAL(21, 278, 24), // "_q_devicePropertyChanged"
QT_MOC_LITERAL(22, 303, 8), // "property"
QT_MOC_LITERAL(23, 312, 5), // "value"
QT_MOC_LITERAL(24, 318, 7), // "isValid"
QT_MOC_LITERAL(25, 326, 14), // "adapterRemoved"
QT_MOC_LITERAL(26, 341, 20), // "requestPairingBluez5"
QT_MOC_LITERAL(27, 362, 17), // "QBluetoothAddress"
QT_MOC_LITERAL(28, 380, 7), // "address"
QT_MOC_LITERAL(29, 388, 30), // "QBluetoothLocalDevice::Pairing"
QT_MOC_LITERAL(30, 419, 13), // "targetPairing"
QT_MOC_LITERAL(31, 433, 17), // "PropertiesChanged"
QT_MOC_LITERAL(32, 451, 9), // "interface"
QT_MOC_LITERAL(33, 461, 18), // "changed_properties"
QT_MOC_LITERAL(34, 480, 22), // "invalidated_properties"
QT_MOC_LITERAL(35, 503, 12), // "QDBusMessage"
QT_MOC_LITERAL(36, 516, 6), // "signal"
QT_MOC_LITERAL(37, 523, 15), // "InterfacesAdded"
QT_MOC_LITERAL(38, 539, 11), // "object_path"
QT_MOC_LITERAL(39, 551, 13), // "InterfaceList"
QT_MOC_LITERAL(40, 565, 25), // "interfaces_and_properties"
QT_MOC_LITERAL(41, 591, 17), // "InterfacesRemoved"
QT_MOC_LITERAL(42, 609, 10), // "interfaces"
QT_MOC_LITERAL(43, 620, 20), // "processPairingBluez5"
QT_MOC_LITERAL(44, 641, 10), // "objectPath"
QT_MOC_LITERAL(45, 652, 6), // "target"
QT_MOC_LITERAL(46, 659, 24) // "pairingDiscoveryTimedOut"

    },
    "QBluetoothLocalDevicePrivate\0Authorize\0"
    "\0QDBusObjectPath\0in0\0in1\0Cancel\0"
    "ConfirmModeChange\0DisplayPasskey\0in2\0"
    "Release\0RequestPasskey\0RequestConfirmation\0"
    "RequestPinCode\0pairingCompleted\0"
    "QDBusPendingCallWatcher*\0PropertyChanged\0"
    "QDBusVariant\0_q_deviceCreated\0device\0"
    "_q_deviceRemoved\0_q_devicePropertyChanged\0"
    "property\0value\0isValid\0adapterRemoved\0"
    "requestPairingBluez5\0QBluetoothAddress\0"
    "address\0QBluetoothLocalDevice::Pairing\0"
    "targetPairing\0PropertiesChanged\0"
    "interface\0changed_properties\0"
    "invalidated_properties\0QDBusMessage\0"
    "signal\0InterfacesAdded\0object_path\0"
    "InterfaceList\0interfaces_and_properties\0"
    "InterfacesRemoved\0interfaces\0"
    "processPairingBluez5\0objectPath\0target\0"
    "pairingDiscoveryTimedOut"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QBluetoothLocalDevicePrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,  119,    2, 0x0a /* Public */,
       6,    0,  124,    2, 0x0a /* Public */,
       7,    1,  125,    2, 0x0a /* Public */,
       8,    3,  128,    2, 0x0a /* Public */,
      10,    0,  135,    2, 0x0a /* Public */,
      11,    1,  136,    2, 0x0a /* Public */,
      12,    2,  139,    2, 0x0a /* Public */,
      13,    1,  144,    2, 0x0a /* Public */,
      14,    1,  147,    2, 0x0a /* Public */,
      16,    2,  150,    2, 0x0a /* Public */,
      18,    1,  155,    2, 0x0a /* Public */,
      20,    1,  158,    2, 0x0a /* Public */,
      21,    2,  161,    2, 0x0a /* Public */,
      24,    0,  166,    2, 0x0a /* Public */,
      25,    1,  167,    2, 0x0a /* Public */,
      26,    2,  170,    2, 0x0a /* Public */,
      31,    4,  175,    2, 0x08 /* Private */,
      37,    2,  184,    2, 0x08 /* Private */,
      41,    2,  189,    2, 0x08 /* Private */,
      43,    2,  194,    2, 0x08 /* Private */,
      46,    0,  199,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::QString,    4,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::UInt, QMetaType::UChar,    4,    5,    9,
    QMetaType::Void,
    QMetaType::UInt, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, QMetaType::UInt,    4,    5,
    QMetaType::QString, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 15,    2,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,    2,    2,
    QMetaType::Void, 0x80000000 | 3,   19,
    QMetaType::Void, 0x80000000 | 3,   19,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,   22,   23,
    QMetaType::Bool,
    QMetaType::Void, 0x80000000 | 3,   19,
    QMetaType::Void, 0x80000000 | 27, 0x80000000 | 29,   28,   30,
    QMetaType::Void, QMetaType::QString, QMetaType::QVariantMap, QMetaType::QStringList, 0x80000000 | 35,   32,   33,   34,   36,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 39,   38,   40,
    QMetaType::Void, 0x80000000 | 3, QMetaType::QStringList,   38,   42,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 29,   44,   45,
    QMetaType::Void,

       0        // eod
};

void QBluetoothLocalDevicePrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QBluetoothLocalDevicePrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->Authorize((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->Cancel(); break;
        case 2: _t->ConfirmModeChange((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->DisplayPasskey((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uchar(*)>(_a[3]))); break;
        case 4: _t->Release(); break;
        case 5: { uint _r = _t->RequestPasskey((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->RequestConfirmation((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 7: { QString _r = _t->RequestPinCode((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->pairingCompleted((*reinterpret_cast< QDBusPendingCallWatcher*(*)>(_a[1]))); break;
        case 9: _t->PropertyChanged((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QDBusVariant(*)>(_a[2]))); break;
        case 10: _t->_q_deviceCreated((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 11: _t->_q_deviceRemoved((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 12: _t->_q_devicePropertyChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QDBusVariant(*)>(_a[2]))); break;
        case 13: { bool _r = _t->isValid();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->adapterRemoved((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1]))); break;
        case 15: _t->requestPairingBluez5((*reinterpret_cast< const QBluetoothAddress(*)>(_a[1])),(*reinterpret_cast< QBluetoothLocalDevice::Pairing(*)>(_a[2]))); break;
        case 16: _t->PropertiesChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2])),(*reinterpret_cast< const QStringList(*)>(_a[3])),(*reinterpret_cast< const QDBusMessage(*)>(_a[4]))); break;
        case 17: _t->InterfacesAdded((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< InterfaceList(*)>(_a[2]))); break;
        case 18: _t->InterfacesRemoved((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 19: _t->processPairingBluez5((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QBluetoothLocalDevice::Pairing(*)>(_a[2]))); break;
        case 20: _t->pairingDiscoveryTimedOut(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusPendingCallWatcher* >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusVariant >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusVariant >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothAddress >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothLocalDevice::Pairing >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 3:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusMessage >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< InterfaceList >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QBluetoothLocalDevice::Pairing >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QBluetoothLocalDevicePrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QBluetoothLocalDevicePrivate.data,
    qt_meta_data_QBluetoothLocalDevicePrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QBluetoothLocalDevicePrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBluetoothLocalDevicePrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QBluetoothLocalDevicePrivate.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int QBluetoothLocalDevicePrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
