/****************************************************************************
** Meta object code from reading C++ file 'agent_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/agent_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'agent_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgBluezAgentAdaptor_t {
    QByteArrayData data[18];
    char stringdata0[1157];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgBluezAgentAdaptor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgBluezAgentAdaptor_t qt_meta_stringdata_OrgBluezAgentAdaptor = {
    {
QT_MOC_LITERAL(0, 0, 20), // "OrgBluezAgentAdaptor"
QT_MOC_LITERAL(1, 21, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 37, 15), // "org.bluez.Agent"
QT_MOC_LITERAL(3, 53, 19), // "D-Bus Introspection"
QT_MOC_LITERAL(4, 73, 946), // "  <interface name=\"org.bluez..."
QT_MOC_LITERAL(5, 926, 9), // "Authorize"
QT_MOC_LITERAL(6, 936, 0), // ""
QT_MOC_LITERAL(7, 937, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(8, 953, 3), // "in0"
QT_MOC_LITERAL(9, 957, 3), // "in1"
QT_MOC_LITERAL(10, 961, 6), // "Cancel"
QT_MOC_LITERAL(11, 968, 17), // "ConfirmModeChange"
QT_MOC_LITERAL(12, 986, 14), // "DisplayPasskey"
QT_MOC_LITERAL(13, 1001, 3), // "in2"
QT_MOC_LITERAL(14, 1005, 7), // "Release"
QT_MOC_LITERAL(15, 1013, 19), // "RequestConfirmation"
QT_MOC_LITERAL(16, 1033, 14), // "RequestPasskey"
QT_MOC_LITERAL(17, 1048, 14) // "RequestPinCode"

    },
    "OrgBluezAgentAdaptor\0D-Bus Interface\0"
    "org.bluez.Agent\0D-Bus Introspection\0"
    "  <interface name=\"org.bluez.Agent\">\n    <method name=\"Release\"/>"
    "\n    <method name=\"RequestPinCode\">\n      <arg direction=\"in\" ty"
    "pe=\"o\"/>\n      <arg direction=\"out\" type=\"s\"/>\n    </method>\n"
    "    <method name=\"RequestPasskey\">\n      <arg direction=\"in\" type"
    "=\"o\"/>\n      <arg direction=\"out\" type=\"u\"/>\n    </method>\n  "
    "  <method name=\"DisplayPasskey\">\n      <arg direction=\"in\" type=\""
    "o\"/>\n      <arg direction=\"in\" type=\"u\"/>\n      <arg direction="
    "\"in\" type=\"y\"/>\n    </method>\n    <method name=\"RequestConfirma"
    "tion\">\n      <arg direction=\"in\" type=\"o\"/>\n      <arg directio"
    "n=\"in\" type=\"u\"/>\n    </method>\n    <method name=\"Authorize\">\n"
    "      <arg direction=\"in\" type=\"o\"/>\n      <arg direction=\"in\" "
    "type=\"s\"/>\n    </method>\n    <method name=\"ConfirmModeChange\">\n"
    "      <arg direction=\"in\" type=\"s\"/>\n    </method>\n    <method n"
    "ame=\"Cancel\"/>\n  </interface>\n\0"
    "Authorize\0\0QDBusObjectPath\0in0\0in1\0"
    "Cancel\0ConfirmModeChange\0DisplayPasskey\0"
    "in2\0Release\0RequestConfirmation\0"
    "RequestPasskey\0RequestPinCode"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgBluezAgentAdaptor[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
       8,   18, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // slots: name, argc, parameters, tag, flags
       5,    2,   58,    6, 0x0a /* Public */,
      10,    0,   63,    6, 0x0a /* Public */,
      11,    1,   64,    6, 0x0a /* Public */,
      12,    3,   67,    6, 0x0a /* Public */,
      14,    0,   74,    6, 0x0a /* Public */,
      15,    2,   75,    6, 0x0a /* Public */,
      16,    1,   80,    6, 0x0a /* Public */,
      17,    1,   83,    6, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 7, QMetaType::QString,    8,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, 0x80000000 | 7, QMetaType::UInt, QMetaType::UChar,    8,    9,   13,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7, QMetaType::UInt,    8,    9,
    QMetaType::UInt, 0x80000000 | 7,    8,
    QMetaType::QString, 0x80000000 | 7,    8,

       0        // eod
};

void OrgBluezAgentAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgBluezAgentAdaptor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->Authorize((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->Cancel(); break;
        case 2: _t->ConfirmModeChange((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->DisplayPasskey((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< uchar(*)>(_a[3]))); break;
        case 4: _t->Release(); break;
        case 5: _t->RequestConfirmation((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 6: { uint _r = _t->RequestPasskey((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->RequestPinCode((*reinterpret_cast< const QDBusObjectPath(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OrgBluezAgentAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_meta_stringdata_OrgBluezAgentAdaptor.data,
    qt_meta_data_OrgBluezAgentAdaptor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgBluezAgentAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezAgentAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgBluezAgentAdaptor.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int OrgBluezAgentAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
