/****************************************************************************
** Meta object code from reading C++ file 'obex_transfer1_bluez5_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/obex_transfer1_bluez5_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'obex_transfer1_bluez5_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgBluezObexTransfer1Interface_t {
    QByteArrayData data[13];
    char stringdata0[136];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgBluezObexTransfer1Interface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgBluezObexTransfer1Interface_t qt_meta_stringdata_OrgBluezObexTransfer1Interface = {
    {
QT_MOC_LITERAL(0, 0, 30), // "OrgBluezObexTransfer1Interface"
QT_MOC_LITERAL(1, 31, 6), // "Cancel"
QT_MOC_LITERAL(2, 38, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(3, 58, 0), // ""
QT_MOC_LITERAL(4, 59, 6), // "Resume"
QT_MOC_LITERAL(5, 66, 7), // "Suspend"
QT_MOC_LITERAL(6, 74, 8), // "Filename"
QT_MOC_LITERAL(7, 83, 4), // "Name"
QT_MOC_LITERAL(8, 88, 7), // "Session"
QT_MOC_LITERAL(9, 96, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(10, 112, 4), // "Size"
QT_MOC_LITERAL(11, 117, 6), // "Status"
QT_MOC_LITERAL(12, 124, 11) // "Transferred"

    },
    "OrgBluezObexTransfer1Interface\0Cancel\0"
    "QDBusPendingReply<>\0\0Resume\0Suspend\0"
    "Filename\0Name\0Session\0QDBusObjectPath\0"
    "Size\0Status\0Transferred"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgBluezObexTransfer1Interface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       6,   32, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   29,    3, 0x0a /* Public */,
       4,    0,   30,    3, 0x0a /* Public */,
       5,    0,   31,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2,
    0x80000000 | 2,
    0x80000000 | 2,

 // properties: name, type, flags
       6, QMetaType::QString, 0x00095001,
       7, QMetaType::QString, 0x00095001,
       8, 0x80000000 | 9, 0x00095009,
      10, QMetaType::ULongLong, 0x00095001,
      11, QMetaType::QString, 0x00095001,
      12, QMetaType::ULongLong, 0x00095001,

       0        // eod
};

void OrgBluezObexTransfer1Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgBluezObexTransfer1Interface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusPendingReply<> _r = _t->Cancel();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->Resume();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->Suspend();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<OrgBluezObexTransfer1Interface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->filename(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 2: *reinterpret_cast< QDBusObjectPath*>(_v) = _t->session(); break;
        case 3: *reinterpret_cast< qulonglong*>(_v) = _t->size(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->status(); break;
        case 5: *reinterpret_cast< qulonglong*>(_v) = _t->transferred(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject OrgBluezObexTransfer1Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_OrgBluezObexTransfer1Interface.data,
    qt_meta_data_OrgBluezObexTransfer1Interface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgBluezObexTransfer1Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezObexTransfer1Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgBluezObexTransfer1Interface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgBluezObexTransfer1Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
