/****************************************************************************
** Meta object code from reading C++ file 'device1_bluez5_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/device1_bluez5_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'device1_bluez5_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgBluezDevice1Interface_t {
    QByteArrayData data[31];
    char stringdata0[329];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgBluezDevice1Interface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgBluezDevice1Interface_t qt_meta_stringdata_OrgBluezDevice1Interface = {
    {
QT_MOC_LITERAL(0, 0, 24), // "OrgBluezDevice1Interface"
QT_MOC_LITERAL(1, 25, 13), // "CancelPairing"
QT_MOC_LITERAL(2, 39, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(3, 59, 0), // ""
QT_MOC_LITERAL(4, 60, 7), // "Connect"
QT_MOC_LITERAL(5, 68, 14), // "ConnectProfile"
QT_MOC_LITERAL(6, 83, 4), // "UUID"
QT_MOC_LITERAL(7, 88, 10), // "Disconnect"
QT_MOC_LITERAL(8, 99, 17), // "DisconnectProfile"
QT_MOC_LITERAL(9, 117, 4), // "Pair"
QT_MOC_LITERAL(10, 122, 7), // "Adapter"
QT_MOC_LITERAL(11, 130, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(12, 146, 7), // "Address"
QT_MOC_LITERAL(13, 154, 5), // "Alias"
QT_MOC_LITERAL(14, 160, 10), // "Appearance"
QT_MOC_LITERAL(15, 171, 7), // "Blocked"
QT_MOC_LITERAL(16, 179, 5), // "Class"
QT_MOC_LITERAL(17, 185, 9), // "Connected"
QT_MOC_LITERAL(18, 195, 4), // "Icon"
QT_MOC_LITERAL(19, 200, 13), // "LegacyPairing"
QT_MOC_LITERAL(20, 214, 16), // "ManufacturerData"
QT_MOC_LITERAL(21, 231, 20), // "ManufacturerDataList"
QT_MOC_LITERAL(22, 252, 8), // "Modalias"
QT_MOC_LITERAL(23, 261, 4), // "Name"
QT_MOC_LITERAL(24, 266, 6), // "Paired"
QT_MOC_LITERAL(25, 273, 4), // "RSSI"
QT_MOC_LITERAL(26, 278, 11), // "ServiceData"
QT_MOC_LITERAL(27, 290, 16), // "ServicesResolved"
QT_MOC_LITERAL(28, 307, 7), // "Trusted"
QT_MOC_LITERAL(29, 315, 7), // "TxPower"
QT_MOC_LITERAL(30, 323, 5) // "UUIDs"

    },
    "OrgBluezDevice1Interface\0CancelPairing\0"
    "QDBusPendingReply<>\0\0Connect\0"
    "ConnectProfile\0UUID\0Disconnect\0"
    "DisconnectProfile\0Pair\0Adapter\0"
    "QDBusObjectPath\0Address\0Alias\0Appearance\0"
    "Blocked\0Class\0Connected\0Icon\0LegacyPairing\0"
    "ManufacturerData\0ManufacturerDataList\0"
    "Modalias\0Name\0Paired\0RSSI\0ServiceData\0"
    "ServicesResolved\0Trusted\0TxPower\0UUIDs"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgBluezDevice1Interface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
      19,   54, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   44,    3, 0x0a /* Public */,
       4,    0,   45,    3, 0x0a /* Public */,
       5,    1,   46,    3, 0x0a /* Public */,
       7,    0,   49,    3, 0x0a /* Public */,
       8,    1,   50,    3, 0x0a /* Public */,
       9,    0,   53,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2,
    0x80000000 | 2,
    0x80000000 | 2, QMetaType::QString,    6,
    0x80000000 | 2,
    0x80000000 | 2, QMetaType::QString,    6,
    0x80000000 | 2,

 // properties: name, type, flags
      10, 0x80000000 | 11, 0x00095009,
      12, QMetaType::QString, 0x00095001,
      13, QMetaType::QString, 0x00095103,
      14, QMetaType::UShort, 0x00095001,
      15, QMetaType::Bool, 0x00095103,
      16, QMetaType::UInt, 0x00095001,
      17, QMetaType::Bool, 0x00095001,
      18, QMetaType::QString, 0x00095001,
      19, QMetaType::Bool, 0x00095001,
      20, 0x80000000 | 21, 0x00095009,
      22, QMetaType::QString, 0x00095001,
      23, QMetaType::QString, 0x00095001,
      24, QMetaType::Bool, 0x00095001,
      25, QMetaType::Short, 0x00095001,
      26, QMetaType::QVariantMap, 0x00095001,
      27, QMetaType::Bool, 0x00095001,
      28, QMetaType::Bool, 0x00095103,
      29, QMetaType::Short, 0x00095001,
      30, QMetaType::QStringList, 0x00095001,

       0        // eod
};

void OrgBluezDevice1Interface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgBluezDevice1Interface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusPendingReply<> _r = _t->CancelPairing();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->Connect();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->ConnectProfile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->Disconnect();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->DisconnectProfile((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->Pair();
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ManufacturerDataList >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<OrgBluezDevice1Interface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QDBusObjectPath*>(_v) = _t->adapter(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->address(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->alias(); break;
        case 3: *reinterpret_cast< ushort*>(_v) = _t->appearance(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->blocked(); break;
        case 5: *reinterpret_cast< uint*>(_v) = _t->classProperty(); break;
        case 6: *reinterpret_cast< bool*>(_v) = _t->connected(); break;
        case 7: *reinterpret_cast< QString*>(_v) = _t->icon(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->legacyPairing(); break;
        case 9: *reinterpret_cast< ManufacturerDataList*>(_v) = _t->manufacturerData(); break;
        case 10: *reinterpret_cast< QString*>(_v) = _t->modalias(); break;
        case 11: *reinterpret_cast< QString*>(_v) = _t->name(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->paired(); break;
        case 13: *reinterpret_cast< short*>(_v) = _t->rSSI(); break;
        case 14: *reinterpret_cast< QVariantMap*>(_v) = _t->serviceData(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->servicesResolved(); break;
        case 16: *reinterpret_cast< bool*>(_v) = _t->trusted(); break;
        case 17: *reinterpret_cast< short*>(_v) = _t->txPower(); break;
        case 18: *reinterpret_cast< QStringList*>(_v) = _t->uUIDs(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<OrgBluezDevice1Interface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setAlias(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setBlocked(*reinterpret_cast< bool*>(_v)); break;
        case 16: _t->setTrusted(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject OrgBluezDevice1Interface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_OrgBluezDevice1Interface.data,
    qt_meta_data_OrgBluezDevice1Interface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgBluezDevice1Interface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgBluezDevice1Interface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgBluezDevice1Interface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgBluezDevice1Interface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 19;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 19;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
