/****************************************************************************
** Meta object code from reading C++ file 'obex_client_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/bluetooth/bluez/obex_client_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'obex_client_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OrgOpenobexClientInterface_t {
    QByteArrayData data[14];
    char stringdata0[217];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OrgOpenobexClientInterface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OrgOpenobexClientInterface_t qt_meta_stringdata_OrgOpenobexClientInterface = {
    {
QT_MOC_LITERAL(0, 0, 26), // "OrgOpenobexClientInterface"
QT_MOC_LITERAL(1, 27, 13), // "CreateSession"
QT_MOC_LITERAL(2, 41, 34), // "QDBusPendingReply<QDBusObject..."
QT_MOC_LITERAL(3, 76, 0), // ""
QT_MOC_LITERAL(4, 77, 3), // "in0"
QT_MOC_LITERAL(5, 81, 21), // "ExchangeBusinessCards"
QT_MOC_LITERAL(6, 103, 19), // "QDBusPendingReply<>"
QT_MOC_LITERAL(7, 123, 3), // "in1"
QT_MOC_LITERAL(8, 127, 3), // "in2"
QT_MOC_LITERAL(9, 131, 15), // "GetCapabilities"
QT_MOC_LITERAL(10, 147, 26), // "QDBusPendingReply<QString>"
QT_MOC_LITERAL(11, 174, 16), // "PullBusinessCard"
QT_MOC_LITERAL(12, 191, 9), // "SendFiles"
QT_MOC_LITERAL(13, 201, 15) // "QDBusObjectPath"

    },
    "OrgOpenobexClientInterface\0CreateSession\0"
    "QDBusPendingReply<QDBusObjectPath>\0\0"
    "in0\0ExchangeBusinessCards\0QDBusPendingReply<>\0"
    "in1\0in2\0GetCapabilities\0"
    "QDBusPendingReply<QString>\0PullBusinessCard\0"
    "SendFiles\0QDBusObjectPath"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OrgOpenobexClientInterface[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   39,    3, 0x0a /* Public */,
       5,    3,   42,    3, 0x0a /* Public */,
       9,    1,   49,    3, 0x0a /* Public */,
      11,    2,   52,    3, 0x0a /* Public */,
      12,    3,   57,    3, 0x0a /* Public */,

 // slots: parameters
    0x80000000 | 2, QMetaType::QVariantMap,    4,
    0x80000000 | 6, QMetaType::QVariantMap, QMetaType::QString, QMetaType::QString,    4,    7,    8,
    0x80000000 | 10, QMetaType::QVariantMap,    4,
    0x80000000 | 6, QMetaType::QVariantMap, QMetaType::QString,    4,    7,
    0x80000000 | 6, QMetaType::QVariantMap, QMetaType::QStringList, 0x80000000 | 13,    4,    7,    8,

       0        // eod
};

void OrgOpenobexClientInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OrgOpenobexClientInterface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { QDBusPendingReply<QDBusObjectPath> _r = _t->CreateSession((*reinterpret_cast< const QVariantMap(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->ExchangeBusinessCards((*reinterpret_cast< const QVariantMap(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<QString> _r = _t->GetCapabilities((*reinterpret_cast< const QVariantMap(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->PullBusinessCard((*reinterpret_cast< const QVariantMap(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->SendFiles((*reinterpret_cast< const QVariantMap(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2])),(*reinterpret_cast< const QDBusObjectPath(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OrgOpenobexClientInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_meta_stringdata_OrgOpenobexClientInterface.data,
    qt_meta_data_OrgOpenobexClientInterface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OrgOpenobexClientInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgOpenobexClientInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OrgOpenobexClientInterface.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgOpenobexClientInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
