/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativebluetoothsocket_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/bluetooth/qdeclarativebluetoothsocket_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativebluetoothsocket_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeclarativeBluetoothSocket_t {
    QByteArrayData data[41];
    char stringdata0[590];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeBluetoothSocket_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeBluetoothSocket_t qt_meta_stringdata_QDeclarativeBluetoothSocket = {
    {
QT_MOC_LITERAL(0, 0, 27), // "QDeclarativeBluetoothSocket"
QT_MOC_LITERAL(1, 28, 14), // "serviceChanged"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 16), // "connectedChanged"
QT_MOC_LITERAL(4, 61, 12), // "errorChanged"
QT_MOC_LITERAL(5, 74, 12), // "stateChanged"
QT_MOC_LITERAL(6, 87, 13), // "dataAvailable"
QT_MOC_LITERAL(7, 101, 10), // "setService"
QT_MOC_LITERAL(8, 112, 29), // "QDeclarativeBluetoothService*"
QT_MOC_LITERAL(9, 142, 7), // "service"
QT_MOC_LITERAL(10, 150, 12), // "setConnected"
QT_MOC_LITERAL(11, 163, 9), // "connected"
QT_MOC_LITERAL(12, 173, 14), // "sendStringData"
QT_MOC_LITERAL(13, 188, 4), // "data"
QT_MOC_LITERAL(14, 193, 16), // "socket_connected"
QT_MOC_LITERAL(15, 210, 19), // "socket_disconnected"
QT_MOC_LITERAL(16, 230, 12), // "socket_error"
QT_MOC_LITERAL(17, 243, 29), // "QBluetoothSocket::SocketError"
QT_MOC_LITERAL(18, 273, 12), // "socket_state"
QT_MOC_LITERAL(19, 286, 29), // "QBluetoothSocket::SocketState"
QT_MOC_LITERAL(20, 316, 16), // "socket_readyRead"
QT_MOC_LITERAL(21, 333, 5), // "error"
QT_MOC_LITERAL(22, 339, 5), // "Error"
QT_MOC_LITERAL(23, 345, 11), // "socketState"
QT_MOC_LITERAL(24, 357, 11), // "SocketState"
QT_MOC_LITERAL(25, 369, 10), // "stringData"
QT_MOC_LITERAL(26, 380, 7), // "NoError"
QT_MOC_LITERAL(27, 388, 18), // "UnknownSocketError"
QT_MOC_LITERAL(28, 407, 21), // "RemoteHostClosedError"
QT_MOC_LITERAL(29, 429, 17), // "HostNotFoundError"
QT_MOC_LITERAL(30, 447, 20), // "ServiceNotFoundError"
QT_MOC_LITERAL(31, 468, 12), // "NetworkError"
QT_MOC_LITERAL(32, 481, 24), // "UnsupportedProtocolError"
QT_MOC_LITERAL(33, 506, 11), // "Unconnected"
QT_MOC_LITERAL(34, 518, 13), // "ServiceLookup"
QT_MOC_LITERAL(35, 532, 10), // "Connecting"
QT_MOC_LITERAL(36, 543, 9), // "Connected"
QT_MOC_LITERAL(37, 553, 5), // "Bound"
QT_MOC_LITERAL(38, 559, 7), // "Closing"
QT_MOC_LITERAL(39, 567, 9), // "Listening"
QT_MOC_LITERAL(40, 577, 12) // "NoServiceSet"

    },
    "QDeclarativeBluetoothSocket\0serviceChanged\0"
    "\0connectedChanged\0errorChanged\0"
    "stateChanged\0dataAvailable\0setService\0"
    "QDeclarativeBluetoothService*\0service\0"
    "setConnected\0connected\0sendStringData\0"
    "data\0socket_connected\0socket_disconnected\0"
    "socket_error\0QBluetoothSocket::SocketError\0"
    "socket_state\0QBluetoothSocket::SocketState\0"
    "socket_readyRead\0error\0Error\0socketState\0"
    "SocketState\0stringData\0NoError\0"
    "UnknownSocketError\0RemoteHostClosedError\0"
    "HostNotFoundError\0ServiceNotFoundError\0"
    "NetworkError\0UnsupportedProtocolError\0"
    "Unconnected\0ServiceLookup\0Connecting\0"
    "Connected\0Bound\0Closing\0Listening\0"
    "NoServiceSet"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeBluetoothSocket[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       5,  102, // properties
       2,  122, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x06 /* Public */,
       3,    0,   80,    2, 0x06 /* Public */,
       4,    0,   81,    2, 0x06 /* Public */,
       5,    0,   82,    2, 0x06 /* Public */,
       6,    0,   83,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,   84,    2, 0x0a /* Public */,
      10,    1,   87,    2, 0x0a /* Public */,
      12,    1,   90,    2, 0x0a /* Public */,
      14,    0,   93,    2, 0x08 /* Private */,
      15,    0,   94,    2, 0x08 /* Private */,
      16,    1,   95,    2, 0x08 /* Private */,
      18,    1,   98,    2, 0x08 /* Private */,
      20,    0,  101,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::Bool,   11,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,    2,
    QMetaType::Void, 0x80000000 | 19,    2,
    QMetaType::Void,

 // properties: name, type, flags
       9, 0x80000000 | 8, 0x0049510b,
      11, QMetaType::Bool, 0x00495103,
      21, 0x80000000 | 22, 0x00495009,
      23, 0x80000000 | 24, 0x00495009,
      25, QMetaType::QString, 0x00495003,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    7,  132,
      24,   24, 0x0,    8,  146,

 // enum data: key, value
      26, uint(QDeclarativeBluetoothSocket::NoError),
      27, uint(QDeclarativeBluetoothSocket::UnknownSocketError),
      28, uint(QDeclarativeBluetoothSocket::RemoteHostClosedError),
      29, uint(QDeclarativeBluetoothSocket::HostNotFoundError),
      30, uint(QDeclarativeBluetoothSocket::ServiceNotFoundError),
      31, uint(QDeclarativeBluetoothSocket::NetworkError),
      32, uint(QDeclarativeBluetoothSocket::UnsupportedProtocolError),
      33, uint(QDeclarativeBluetoothSocket::Unconnected),
      34, uint(QDeclarativeBluetoothSocket::ServiceLookup),
      35, uint(QDeclarativeBluetoothSocket::Connecting),
      36, uint(QDeclarativeBluetoothSocket::Connected),
      37, uint(QDeclarativeBluetoothSocket::Bound),
      38, uint(QDeclarativeBluetoothSocket::Closing),
      39, uint(QDeclarativeBluetoothSocket::Listening),
      40, uint(QDeclarativeBluetoothSocket::NoServiceSet),

       0        // eod
};

void QDeclarativeBluetoothSocket::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeBluetoothSocket *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->serviceChanged(); break;
        case 1: _t->connectedChanged(); break;
        case 2: _t->errorChanged(); break;
        case 3: _t->stateChanged(); break;
        case 4: _t->dataAvailable(); break;
        case 5: _t->setService((*reinterpret_cast< QDeclarativeBluetoothService*(*)>(_a[1]))); break;
        case 6: _t->setConnected((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->sendStringData((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->socket_connected(); break;
        case 9: _t->socket_disconnected(); break;
        case 10: _t->socket_error((*reinterpret_cast< QBluetoothSocket::SocketError(*)>(_a[1]))); break;
        case 11: _t->socket_state((*reinterpret_cast< QBluetoothSocket::SocketState(*)>(_a[1]))); break;
        case 12: _t->socket_readyRead(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeBluetoothService* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeBluetoothSocket::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothSocket::serviceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothSocket::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothSocket::connectedChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothSocket::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothSocket::errorChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothSocket::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothSocket::stateChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QDeclarativeBluetoothSocket::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeBluetoothSocket::dataAvailable)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeBluetoothService* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeBluetoothSocket *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QDeclarativeBluetoothService**>(_v) = _t->service(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->connected(); break;
        case 2: *reinterpret_cast< Error*>(_v) = _t->error(); break;
        case 3: *reinterpret_cast< SocketState*>(_v) = _t->state(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->stringData(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeBluetoothSocket *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setService(*reinterpret_cast< QDeclarativeBluetoothService**>(_v)); break;
        case 1: _t->setConnected(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->sendStringData(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeBluetoothSocket::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeBluetoothSocket.data,
    qt_meta_data_QDeclarativeBluetoothSocket,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeBluetoothSocket::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeBluetoothSocket::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeBluetoothSocket.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeBluetoothSocket::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeBluetoothSocket::serviceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeBluetoothSocket::connectedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeBluetoothSocket::errorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeBluetoothSocket::stateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeBluetoothSocket::dataAvailable()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
